<?php
/*
 * ZFS.php
 *
 * part of pfSense (https://www.pfsense.org)
 * Copyright (c) 2013-2023 Rubicon Communications, LLC (Netgate)
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

Namespace Netgate\Services\ZFS;

final class ZFS {
	private $pools = [];

	private $libzfs = null;

	function __construct(LibZFS $libzfs = null) {
		if (is_null($libzfs) || (!($libzfs instanceof LibZFS))) {
			$libzfs = new LibZFS();
		}

		$this->libzfs = $libzfs;

		$this->_initPools();
	}

	public function getLibzfs() {
		return $this->libzfs;
	}

	public function setLibzfs(LibZFS $libzfs) {
		$this->libzfs = $libzfs;

		return $this;
	}

	public function getPools() {
		return $this->getPoolObjects();
	}

	public function hasPools() {
		return (!empty($this->getPools()) && (count($this->getPools()) > 0));
	}

	public function getPoolByGuid($guid) {
		return $this->_getPoolBy('guid', $guid);
	}

	public function getPoolById($id) {
		return $this->_getPoolBy('id', $id);
	}

	private function getPoolObjects() {
		if (empty($this->pools)) {
			$this->_initPools();
		}

		return $this->pools;
	}

	private function _getPoolBy($key, $value) {
		foreach ($this->getPools() as $guid => $pool) {
			if ($pool->getProperty($key) === $value) {
				return $pool;
			}
		}

		return false;
	}

	private function _initPools() {
		$pools = $this->getLibzfs()->getZfsData();

		foreach ($pools as $pool) {
			$pool = $this->_getNewPoolObject($pool);

			array_push($this->pools, $pool);
		}
	}

	private function _getNewPoolObject($pool) {
		return (new Pool($pool));
	}
}

?>