<?php
/*
 * system_be.php
 *
 * part of pfSense plus (https://www.netgate.com)
 * Copyright (c) 2022-2023 Rubicon Communications, LLC (Netgate)
 * All rights reserved.
 */

##|+PRIV
##|*IDENT=page-system-bootenvironments
##|*NAME=System: Boot Environments
##|*DESCR=Allow access to the 'System: Boot Environments' page.
##|*MATCH=system_be*
##|-PRIV

// pfSense includes
require_once('functions.inc');
require_once('guiconfig.inc');

// BE includes
require_once('system_be.inc');

// should we display this page?
$has_be_support = be_bootenv_check();

/* handle post actions */
if ($_POST && $has_be_support) {
	$be_name = $_POST['be'];
	switch ($_POST['act']) {
		case 'activate':
			$input_errors = be_bootenv_do_activate($be_name);
			break;
		case 'delete':
			$input_errors = be_bootenv_do_delete($be_name);
			break;
		case 'quick':
			$input_errors = be_bootenv_do_quick();
			break;
		default:
			/* something isn't right, bailout... */
			header('Location: /system_be.php');
			break;
	}
}

$pgtitle = array(gettext('System'), gettext('Boot Environments'));
$pglinks = array('', '@self');

include('head.inc');

if (!$has_be_support) {
	print_info_box(gettext('This system does not support ZFS Boot Environments.'), 'warning', null);
	goto foot;
}

if (!empty($input_errors)) {
	print_input_errors($input_errors);
}
?>

<div class="panel panel-default">
	<div class="panel-heading"><h2 class="panel-title"><?=gettext('Boot Environments')?></h2></div>
	<div class="panel-body table-responsive">
		<table class="table table-hover table-striped table-condensed">
			<thead>
				<tr>
					<th><!--status icon--></th>
					<th><?=gettext('Name')?></th>
					<th><?=gettext('Base Version')?></th>
					<th><?=gettext('Created')?></th>
					<th><?=gettext('Last Booted')?></th>
					<th><?=gettext('Space')?></th>
					<th><?=gettext('Description')?></th>
					<th><?=gettext('Actions')?></th>
				</tr>
			</thead>
			<tbody>
<?php foreach (be_bootenv_list() as $be_name => $props): ?>
				<tr id="<?=htmlspecialchars($be_name)?>" style="cursor:default;">
					<td><?=$props['status_icon']?></td>
					<td><?=htmlspecialchars($be_name)?></td>
					<td><?=htmlspecialchars($props['version'])?></td>
					<td><?=htmlspecialchars($props['creation_nice'])?></td>
					<td><?=htmlspecialchars($props['lastbooted_nice'])?></td>
					<td><?=htmlspecialchars($props['used_nice'])?></td>
					<td><?=htmlspecialchars($props['descr'])?></td>
					<td style="cursor: pointer;">
						<a class="fa <?=htmlspecialchars($props['activate_icon'])?>" title="<?=htmlspecialchars($props['activate_title'])?>" href="<?="?act=activate&be={$be_name}"?>" usepost></a>
						<a class="fa fa-pencil" title="<?=gettext('Edit Boot Environment')?>" href="<?="system_be_edit.php?be={$be_name}"?>"></a>
						<a class="fa fa-clone" title="<?=gettext('Clone Boot Environment')?>" href="<?="system_be_edit.php?from={$be_name}"?>" usepost></a>
						<a class="fa fa-play-circle" title="<?=gettext('Activate One Time and Reboot')?>" href="<?="/diag_reboot.php?be={$be_name}"?>" usepost></a>
						<a class="fa fa-trash text-danger" title="<?=gettext('Delete Boot Environment')?>" href="<?="?act=delete&be={$be_name}"?>" usepost></a>
					</td>
				</tr>
<?php endforeach; ?>
			</tbody>
		</table>
	</div>
</div>
<nav class="action-buttons">
	<a href="?act=quick" class="btn btn-success btn-sm" usepost>
		<i class="fa fa-bolt icon-embed-btn"></i>
		<?=gettext('Quick Create')?>
	</a>
	<a href="system_be_edit.php" class="btn btn-warning btn-sm" usepost>
		<i class="fa fa-plus icon-embed-btn"></i>
		<?=gettext('Create')?>
	</a>
</nav>

<?php
foot:
include('foot.inc');
