<?php
/*
 * zfs.widget.php
 *
 * part of pfSense (https://www.pfsense.org)
 * Copyright (c) 2021-2023 Rubicon Communications, LLC (Netgate)
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

 // Composer autoloader
require_once('vendor/autoload.php');

// pfSense includes
require_once('guiconfig.inc');

// Widget includes
require_once('/usr/local/www/widgets/include/zfs.inc');

// Randomly invalidate the cache, 25% chance.
zfs_cache_invalidate(false, 0.25);

// Are we handling an ajax refresh?
if (isset($_POST['ajax'])) {
	print(zfs_compose_widget_table($widget_config));

	// We are done here...
	exit();
}

?>

	<div class="table-responsive">
		<table id="<?=htmlspecialchars($widgetkey)?>-table" class="table table-striped table-hover table-condensed">
			<?=zfs_compose_widget_table()?>
		</table>
	</div>

	<script type="text/javascript">
	//<![CDATA[
	events.push(function(){
		let cookieName = <?=json_encode("treegrid-{$widgetkey}")?>;

		// Callback function called by refresh system when data is retrieved
		function zfs_callback(s) {
			var tree = $(<?=json_encode("#{$widgetkey}-table")?>);

			tree.removeData();

			tree.html(s);

			initTreegrid(true);
		}

		// POST data to send via AJAX
		var postdata = {
			ajax: "ajax",
			widgetkey: <?=json_encode($widgetkey)?>
		};

		// Create an object defining the widget refresh AJAX call
		var zfsObject = new Object();
		zfsObject.name = "zfs";
		zfsObject.url = "/widgets/widgets/zfs.widget.php";
		zfsObject.callback = zfs_callback;
		zfsObject.parms = postdata;
		zfsObject.freq = 1;

		// Register the AJAX object
		register_ajax(zfsObject);

		function initTreegrid(isAjax) {
			var tree = $(<?=json_encode("#{$widgetkey}-table")?>);

			if (!isAjax) {
				$.removeCookie(cookieName);
				
				tree.removeData();
			}

			tree.treegrid({
				expanderExpandedClass: 'fa fa fa-chevron-down',
				expanderCollapsedClass: 'fa fa fa-chevron-right',
				initialState: 'collapsed',
				saveStateName: cookieName,
				saveState: true
			});
		}

		initTreegrid(false);
	});
	//]]>
	</script>
